@@ -13,6 +13,7 @@ import os
 from os.path import join, basename, dirname, isdir
 import numpy as np
 from shapely.geometry import box
+import pandas as pd
 import geopandas as gpd
 import xarray as xr
 import dask
@@ -28,22 +29,13 @@ from scipy import ndimage
 import tempfile
 import pyproj
 import logging
+import rioxarray
 
-from . import gis_utils, HAS_PCRASTER
+from . import gis_utils, _compat
 
 logger = logging.getLogger(__name__)
 XDIMS = ("x", "longitude", "lon", "long")
 YDIMS = ("y", "latitude", "lat")
-FILL_VALUE_NAMES = ("_FillValue", "missing_value", "fill_value", "nodata", "nodatavals")
-UNWANTED_RIO_ATTRS = (
-    "nodatavals",
-    "crs",
-    "is_tiled",
-    "res",
-    "transform",
-    "scales",
-    "offsets",
-)
 GEO_MAP_COORD = "spatial_ref"
 
 
@@ -188,10 +180,7 @@ class XGeoBase(object):
     @property
     def crs(self):
         """Return Coordinate Reference System as :py:meth:`pyproj.CRS` object."""
-        if self.get_attrs("crs_wkt") is None:
-            self.set_crs()
-        crs = self.get_attrs("crs_wkt")
-        return CRS.from_wkt(crs) if crs is not None else crs
+        return self._obj.rio.crs
 
     @property
     def x_dim(self):
@@ -294,7 +283,7 @@ class XGeoBase(object):
                     except:
                         pass
         if input_crs is not None:
-            self.set_attrs(crs_wkt=input_crs.wkt)
+            self._obj.rio.write_crs(input_crs, inplace=True)
 
     def reset_spatial_dims_attrs(self):
         """Reset spatial dimension names and attributes to make CF-compliant
@@ -409,8 +398,8 @@ class XRasterBase(XGeoBase):
             dims = tuple(extra_dims) + dims
             self.set_attrs(dim0=extra_dims[0])
         elif len(extra_dims) == 0:
-            self.set_attrs(dim0=None)
-        else:
+            self._obj.coords[GEO_MAP_COORD].attrs.pop("dim0", None)
+        elif len(extra_dims) > 1:
             raise ValueError("Only 2D and 3D data arrays supported.")
         if isinstance(self._obj, xr.Dataset):
             check = np.all([self._obj[name].dims == dims for name in self.vars])
@@ -453,6 +442,45 @@ class XRasterBase(XGeoBase):
             and np.logical_and.reduce((w <= w1, s <= s1, e >= e1, n >= n1))
         )
 
+    def gdal_compliant(self, rename_dims=True, force_sn=False):
+        """Updates attributes to get GDAL compliant NetCDF files.
+
+        Arguments
+        ----------
+        rename_dims: bool, optional
+            If True, rename x_dim and y_dim to standard names depending on the CRS
+            (x/y for projected and lat/lon for geographic).
+        force_sn: bool, optional
+            If True, forces the dataset to have South -> North orientation.
+
+        Returns
+        -------
+        ojb_out: xr.Dataset or xr.DataArray
+            GDAL compliant object
+        """
+        obj_out = self._obj
+        crs = obj_out.raster.crs
+        if (
+            obj_out.raster.res[1] < 0 and force_sn
+        ):  # write data with South -> North orientation
+            obj_out = obj_out.raster.flipud()
+        x_dim, y_dim, x_attrs, y_attrs = gis_utils.axes_attrs(crs)
+        if rename_dims:
+            obj_out = obj_out.rename(
+                {obj_out.raster.x_dim: x_dim, obj_out.raster.y_dim: y_dim}
+            )
+        else:
+            x_dim = obj_out.raster.x_dim
+            y_dim = obj_out.raster.y_dim
+        obj_out[x_dim].attrs.update(x_attrs)
+        obj_out[y_dim].attrs.update(y_attrs)
+        obj_out = obj_out.drop_vars(["spatial_ref"], errors="ignore")
+        obj_out.rio.write_crs(crs, inplace=True)
+        obj_out.rio.write_transform(obj_out.raster.transform, inplace=True)
+        obj_out.raster.set_spatial_dims()
+
+        return obj_out
+
     def transform_bounds(self, dst_crs, densify_pts=21):
         """Transform bounds from object to destination CRS.
 
@@ -763,6 +791,52 @@ class XRasterBase(XGeoBase):
 
         return ds_out
 
+    def reclassify(self, reclass_table: pd.DataFrame, method: str = "exact"):
+        """Reclass columns in df from raster map (DataArray).
+
+        Arguments
+        ---------
+        reclass_table : pd.DataFrame
+            Tables with parameter names and values in columns and values in obj as index.
+        method : str, optional
+            Reclassification method. For now only 'exact' for one-on-one cell value mapping.
+
+        Returns
+        -------
+        ds_out: xr.Dataset
+            Output dataset with a variable for each column in reclass_table.
+        """
+        # Exact reclass method
+        def reclass_exact(x, ddict):
+            return np.vectorize(ddict.get)(x, np.nan)
+
+        da = self._obj.copy()
+        ds_out = xr.Dataset(coords=da.coords)
+
+        keys = reclass_table.index.values
+        params = reclass_table.columns
+        # limit dtypes to avoid gdal errors downstream
+        ddict = {"float64": np.float32, "int64": np.int32}
+        dtypes = {
+            c: ddict.get(str(reclass_table[c].dtype), reclass_table[c].dtype)
+            for c in reclass_table.columns
+        }
+        reclass_table = reclass_table.astype(dtypes)
+        # apply for each parameter
+        for param in params:
+            values = reclass_table[param].values
+            d = dict(zip(keys, values))
+            da_param = xr.apply_ufunc(
+                reclass_exact,
+                da,
+                dask="parallelized",
+                output_dtypes=[values.dtype],
+                kwargs={"ddict": d},
+            )
+            da_param.attrs.update(_FillValue=np.nan)
+            ds_out[param] = da_param
+        return ds_out
+
     def clip_bbox(self, bbox, align=None, buffer=0):
         """Clip object based on a bounding box.
 
@@ -960,9 +1034,16 @@ class XRasterBase(XGeoBase):
         """
         gdf1 = gdf.copy()
         gdf1["mask"] = np.full(gdf.index.size, (not invert), dtype=np.uint8)
-        return self.rasterize(
-            gdf1, col_name="mask", all_touched=all_touched, nodata=invert, **kwargs
-        ).astype(bool)
+        da_out = self.rasterize(
+            gdf1,
+            col_name="mask",
+            all_touched=all_touched,
+            nodata=np.uint8(invert),
+            **kwargs,
+        )
+        # remove nodata value before converting to boolean
+        da_out.attrs.pop("_FillValue", None)
+        return da_out.astype(bool)
 
     def vector_grid(self):
         """Return a geopandas GeoDataFrame with a box for each grid cell."""
@@ -1205,14 +1286,15 @@ class RasterDataArray(XRasterBase):
     @property
     def nodata(self):
         """Nodata value of the DataArray."""
-        _attrs = self._obj.attrs
-        _encoding = self._obj.encoding
-        if _attrs.get("_FillValue", None) is None:
-            self.set_nodata()
         # first check attrs, then encoding
-        return _attrs.get("_FillValue", _encoding.get("_FillValue", None))
+        nodata = self._obj.rio.nodata
+        if nodata is None:
+            nodata = self._obj.rio.encoded_nodata
+            if nodata is not None:
+                self.set_nodata(nodata)
+        return nodata
 
-    def set_nodata(self, nodata=None):
+    def set_nodata(self, nodata=None, logger=logger):
         """Set the nodata value as CF compliant attribute of the DataArray.
 
         Arguments
@@ -1222,23 +1304,17 @@ class RasterDataArray(XRasterBase):
             If the nodata property and argument are both None, the _FillValue
             attribute will be removed.
         """
-        dtype = str(self._obj.dtype)
         if nodata is None:
-            for name in FILL_VALUE_NAMES:
-                if name in self._obj.attrs:
-                    # remove from attrs and set property
-                    nodata = self._obj.attrs.pop(name)
-                    if isinstance(nodata, tuple):
-                        nodata = nodata[0]
-                    try:
-                        nodata = getattr(np, dtype)(nodata)
-                        break
-                    except ValueError:
-                        nodata = None
-        if nodata is not None:  # keep attribute property
-            self._obj.attrs.update({"_FillValue": nodata})
-        elif "_FillValue" in self._obj.attrs:
-            self._obj.attrs.pop("_FillValue")
+            nodata = self._obj.rio.nodata
+            if nodata is None:
+                nodata = self._obj.rio.encoded_nodata
+        # Only numerical nodata values are supported
+        if np.issubdtype(type(nodata), np.number):
+            self._obj.rio.set_nodata(nodata, inplace=True)
+            self._obj.rio.write_nodata(nodata, inplace=True)
+        else:
+            logger.warning("No numerical nodata value found, skipping set_nodata")
+            self._obj.attrs.pop("_FillValue", None)
 
     def mask_nodata(self, fill_value=np.nan):
         """Mask nodata values with fill_value (default np.nan).
@@ -1257,7 +1333,7 @@ class RasterDataArray(XRasterBase):
         if self.nodata is not None:
             da_masked = self._obj.where(mask != 0, self.nodata)
         else:
-            logger.warn("Nodata value missing, skipping mask")
+            logger.warning("Nodata value missing, skipping mask")
             da_masked = self._obj
         return da_masked
 
@@ -1521,18 +1597,21 @@ class RasterDataArray(XRasterBase):
         if not mask.any() or mask.all():
             return src_data
         if method == "rio_idw":  # NOTE: modifies src_data inplace
+            # NOTE this method might also extrapolate
             interp_data = rasterio.fill.fillnodata(src_data.copy(), mask, **kwargs)
         else:
+            # get valid cells D4-neighboring nodata cells to setup triangulation
+            valid = np.logical_and(mask, ndimage.binary_dilation(~mask))
             xs, ys = np.meshgrid(self.xcoords.values, self.ycoords.values)
-            interp_data = griddata(
-                points=(xs[mask], ys[mask]),
-                values=src_data[mask],
-                xi=(xs, ys),
+            # interpolate data at nodata cells only
+            interp_data = src_data.copy()
+            interp_data[~mask] = griddata(
+                points=(xs[valid], ys[valid]),
+                values=src_data[valid],
+                xi=(xs[~mask], ys[~mask]),
                 method=method,
                 fill_value=self.nodata,
             )
-            # required because in some cases valid values are set to nodata?!
-            interp_data = np.where(mask, src_data, interp_data)
         return interp_data
 
     def interpolate_na(self, method: str = "nearest", **kwargs):
@@ -1633,7 +1712,7 @@ class RasterDataArray(XRasterBase):
             count = da_out[dim0].size
             da_out = da_out.sortby(dim0)
         # write
-        if driver.lower() == "pcraster" and HAS_PCRASTER:
+        if driver.lower() == "pcraster" and _compat.HAS_PCRASTER:
             for i in range(count):
                 if dim0:
                     bname = basename(raster_path).split(".")[0]
@@ -1845,6 +1924,28 @@ class RasterDataset(XRasterBase):
                 )
         return ds
 
+    def interpolate_na(self, method: str = "nearest", **kwargs):
+        """Interpolate missing data
+
+        Arguments
+        ----------
+        method: {'linear', 'nearest', 'cubic', 'rio_idw'}, optional
+            {'linear', 'nearest', 'cubic'} use :py:meth:`scipy.interpolate.griddata`;
+            'rio_idw' applies inverse distance weighting based on :py:meth:`rasterio.fill.fillnodata`.
+        **kwargs
+            Additional key-word arguments are passed to :py:meth:`rasterio.fill.fillnodata`,
+            only used in combination with `method='rio_idw'`
+
+        Returns
+        -------
+        xarray.Dataset
+            Filled object
+        """
+        ds_out = xr.Dataset(attrs=self._obj.attrs)
+        for var in self.vars:
+            ds_out[var] = self._obj[var].raster.interpolate_na(method=method, **kwargs)
+        return ds_out
+
     def reproject_like(self, other, method="nearest"):
         """Reproject a Dataset object to match the resolution, projection,
         and region of ``other``.
@@ -1965,7 +2066,7 @@ class RasterDataset(XRasterBase):
         if not isdir(root):
             os.makedirs(root)
         with tempfile.TemporaryDirectory() as tmpdir:
-            if driver == "PCRaster" and HAS_PCRASTER:
+            if driver == "PCRaster" and _compat.HAS_PCRASTER:
                 clone_path = gis_utils.write_clone(
                     tmpdir,
                     gdal_transform=self.transform.to_gdal(),
